note
	description: "[
		Three versions illustrating how DbC (e.g., addition of an appropriate precondition) 
		is useful in developing proper interaction between features 
		(i.e., {DATABASE}.add_name and {UTILITIES}.search).
		
		V1: DATABASE_V1 passes an unsorted array to binary search in UTILITIES_V1.
				External behaviour is incorrect, but no contract violation occurs. 
				
		V2: DATABASE_V2 passes an unsorted array to binary search in UTILITIES_V2.
				{UTILITIES_V2}.search declares a precondition `array_sorted`.
				Contract violation occurs, which forces the client to change (see DATABASE_V3). 
				
		V3: DATABASE_V3 passes a sorted array to binary search in TILITIES_V3.
				{UTILITIES_V3}.search declares a precondition `array_sorted`.
				The external behaviour is correct. Contract violation does not occur.
				This version may now be finalized wth contract checking turned off.
	]"
	author: "Jackie Wang"
	date: "$Date$"
	revision: "$Revision$"

class
	ROOT

inherit

	ARGUMENTS_32

	ES_SUITE -- testing via ESpec

create
	make

feature -- Mode

	is_test_mode: BOOLEAN = False

feature {NONE} -- Initialization

	make
			-- Run app
		local
			input: STRING
			done: BOOLEAN
			cmd_line_option: STRING
			database: DATABASE
		do
			if is_test_mode then
				add_test (create {TEST_UTILITIES}.make) --suite of tests
				show_browser
				run_espec
			else
				if not (argument_count = 1 and then
							(	argument_array[1] ~ "-v1" or else
							 	argument_array[1] ~ "-v2" or else
							 	argument_array[1] ~ "-v3"	))
				then
					io.put_string ("Usage: why-dbc-useful -v1 | -v2 | -v3%N")
				else
					if argument_array[1] ~ "-v1" then
						-- wrong implementation, no contract, wrong behaviour
						create {DATABASE_V1} database.make
					elseif argument_array[1] ~ "-v2" then
						-- wrong implementation, added preconditin, contract violation
						create {DATABASE_V2} database.make
					else
						-- correct implementation, added precondition, no contract violation
						create {DATABASE_V3} database.make
					end

					from
					until
						done
					loop
						io.put_string ("Enter a name, or `done` to start inquiring: ")
						io.read_line
						input := io.last_string
						if input ~ "done" then
							done := True
						else
							database.add_name (input.twin)
						end
					end

					across
						database.data is l_n
					loop
						io.put_string (l_n + "  ")
					end
					io.put_string ("%N")

					from
						done := false
					until
						done
					loop
						io.put_string ("Enter a name, or `quit` to stop inquiring: ")
						io.read_line
						input := io.last_string
						if input ~ "quit" then
							done := True
						else
							io.put_string (
								if database.data_exists (input)
									then input + " exists!"
									else input + " does not exist!"
								end
							)
							io.put_string ("%N")
						end
					end
				end
			end
		end

end
